package org.bdware.server.action;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.ContractResult;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.sc.util.JsonUtil;
import org.bdware.server.nodecenter.CMNode;
import org.bdware.server.nodecenter.ContractExecutor;
import org.bdware.server.nodecenter.MultiPointContractInfo;
import org.bdware.server.nodecenter.NodeCenterActions;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class RequestAllExecutor implements ContractExecutor {
    private static final Logger LOGGER = LogManager.getLogger(RequestAllExecutor.class);
    MultiPointContractInfo info;
    int resultCount;

    public RequestAllExecutor(MultiPointContractInfo minfo, int count) {
        info = minfo;
        resultCount = count;
    }

    public ResultCallback createResultCallback(
            final String requestID, final ResultCallback originalCb, int count, String contractID) {
        return new Collector(requestID, originalCb, count, contractID);
    }

    //only for test ADSP
//    public ResultCallback createResultCallback(
//            final String requestID, final ResultCallback originalCb, int count,String contractID,String seq) {
//        ResultCallback collector = new Collector(requestID, originalCb, count,contractID,seq);
//        return collector;
//    }

    public void sendRequest(String req, ResultCallback collector) {
        List<String> nodes = info.members;
        for (String node : nodes) {
            CMNode cmNode = NodeCenterActions.nodeInfos.get(node);
            if (cmNode == null) {
                collector.onResult(
                        "{\"status\":\"Error\",\"result\":\"node "
                                + node
                                + " offline\",\"action\":\"onExecuteContractTrustfully\"}");
            } else {
                cmNode.connection.controller.sendMsg(req);
            }
        }
    }

    @Override
    public void execute(String requestID, ResultCallback rc, String req) {
        JsonObject jo2 = JsonParser.parseString(req).getAsJsonObject();
        String id = jo2.get("contractID").getAsString();

        //only for test ADSP
//        ResultCallback collector = createResultCallback(requestID, rc, resultCount,id,jo2.get("seq").getAsString());

        ResultCallback collector = createResultCallback(requestID, rc, resultCount, id);

        //TODO NC.sync?有问题？
        NodeCenterActions.sync.sleep(requestID, collector);
        sendRequest(req, collector);
    }

    static class Collector extends ResultCallback {
        String contractID;
        int total;
        String requestID;
        AtomicInteger count = new AtomicInteger(0);


        //only for testADSP
//        boolean flag = true;  //true表示还没在文件中记录这次调用结果
//        String seq;
//        Map<String,Integer> res = new ConcurrentHashMap<String,Integer>();


        //only for test ADSP
//        public Collector(String reqID, ResultCallback callback, int c,String id,String seq2) {
//            total = c;
//            requestID = reqID;
//            commiter = callback;
//            contractID = id;
//            seq = seq2;
//        }
        ResultCallback committer;

        public Collector(String reqID, ResultCallback callback, int c, String id) {
            total = c;
            requestID = reqID;
            committer = callback;
            contractID = id;
        }

        @Override
        public void onResult(String str) {
            JsonObject obj = JsonParser.parseString(str).getAsJsonObject();
            obj.remove("action");
            obj.addProperty("action", "onExecuteContractTrustfully");
            obj.addProperty("seqNum", count.getAndIncrement());
            obj.addProperty("total", total);

            //NodeCenterActions.recoverMap.get(nodePubKey).get(contractID).lastExeSeq++;

            if (committer != null)
                committer.onResult(obj.toString());
            if (count.get() < total) {
                NodeCenterActions.sync.sleepWithTimeout(requestID, this, 10);
            }

            String name = "null";
            if (obj.has("nodeID")) {
                String nodePubKey = obj.get("nodeID").getAsString();
                CMNode node = NodeCenterActions.nodeInfos.get(nodePubKey);
                name = node.nodeName;
            }
            LOGGER.debug("收到第 " + count.get() + " 个节点  " + name + "  回复 : " + obj);

            if (obj.has("data")) {
                String data = obj.get("data").getAsString();
                ContractResult cr = JsonUtil.fromJson(data, ContractResult.class);

                //only for testADSP
//                if(TestADSP.getFlag()){
//                    String r = cr.result;
//                    if(!res.containsKey(r)){
//                        res.put(r,0);
//                    }
//                    int former = res.get(r);
//                    res.put(r,++former);
//                    for(String s : res.keySet()){
//                        if(res.get(s) > total / 2 && flag){
//                            //写入文件
//                            File file = new File(TestADSP.resultPath);
//                            synchronized (file){
//                                try {
//                                    FileWriter fw = new FileWriter(file, true);
//                                    PrintWriter pw = new PrintWriter(fw);
//                                    pw.println(seq + " " + r);
//                                    pw.flush();
//                                    fw.flush();
//                                    pw.close();
//                                    fw.close();
//                                } catch (IOException e) {
//                                    e.printStackTrace();
//                                }
//                            }
//                            flag = false;
//                        }
//                    }
//                }


/*                if (cr.status == ContractResult.Status.Error) {   //TODO   规范Status的使用       改成 ==Statuc.Error才恢复
                    String nodePubKey = obj.get("nodeID").getAsString();
                    if(NodeCenterActions.recoverMap.containsKey(nodePubKey)){
                        ContractRecord record = NodeCenterActions.recoverMap.get(nodePubKey).get(contractID);
                        if(record.recoverFlag == RecoverFlag.Fine){
                            record.recoverFlag = RecoverFlag.ToRecover;
                            MasterActions.restartContracts(nodePubKey);
                        }
                    }
                }*/
            }
        }
    }
}
