package org.bdware.server.action;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.server.nodecenter.CMNode;
import org.bdware.server.nodecenter.ContractExecutor;
import org.bdware.server.nodecenter.MultiPointContractInfo;
import org.bdware.server.nodecenter.NodeCenterActions;

import java.util.concurrent.atomic.AtomicInteger;

public class RequestOnceExecutor implements ContractExecutor {
    private static final Logger LOGGER = LogManager.getLogger(RequestOnceExecutor.class);
    MultiPointContractInfo info;
    AtomicInteger order = new AtomicInteger(0);

    public RequestOnceExecutor(MultiPointContractInfo minfo) {
        info = minfo;
    }

    @Override
    public void execute(String requestID, ResultCallback rc, String req) {
        JsonObject jo2 = (new JsonParser()).parse(req).getAsJsonObject();
        String contractID = jo2.get("contractID").getAsString();

        ResultCallback cb =
                new ResultCallback() {
                    @Override
                    public void onResult(String str) {
                        LOGGER.debug(str);
                        JsonObject jo = (new JsonParser()).parse(str).getAsJsonObject();
                        jo.remove("action");
                        jo.addProperty("action", "onExecuteResult");
                        LOGGER.debug(jo.toString());
                        rc.onResult(jo.toString());
                    }
                };
        NodeCenterActions.sync.sleep(requestID, cb);
        for (int i = 0; i < info.members.size(); i++) {
            int size = info.members.size();
            String nodeID = info.members.get(order.incrementAndGet() % size);
            CMNode cmNode = NodeCenterActions.nodeInfos.get(nodeID);

            if (cmNode != null) {
                cmNode.connection.controller.sendMsg(req);
                return;
            }
        }
        rc.onResult(
                "{\"status\":\"Error\",\"result\":\"all nodes "
                        + " offline\",\"action\":\"onExecuteContract\"}");
    }
}
