package org.bdware.server.nodecenter;

import com.google.gson.JsonObject;
import com.google.gson.reflect.TypeToken;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.bean.OtherNCInfo;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.sc.util.JsonUtil;
import org.bdware.server.action.Action;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.zz.gmhelper.SM2Util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class NCClientActions {
    private static final Logger LOGGER = LogManager.getLogger(NCClientActions.class);
    private final NCClientHandler handler;

    public NCClientActions(NCClientHandler h) {
        handler = h;
    }

    @Action
    public void onSessionID(JsonObject json, ResultCallback cb) {
        LOGGER.info("[NCClientActions] onSessionID : ");

        Map<String, String> map = new HashMap<>();
        LOGGER.debug("[onSessionID]:" + json.toString());
        map.put("action", "getNCConnectPermission");
        map.put("id", OtherNCProxy.instance.sm2.getPublicKeyStr());
        byte[] signature = "no signature".getBytes();
        try {
            signature = SM2Util.sign(OtherNCProxy.instance.sm2.getPrivateKeyParameter(), (OtherNCProxy.instance.sm2.getPublicKeyStr() + json.get("session").getAsString())
                    .getBytes());
        } catch (CryptoException e) {
            e.printStackTrace();
        }
        map.put("signature", ByteUtils.toHexString(signature));
        sendMsg(JsonUtil.toJson(map));
    }

    @Action
    public void onGetNCConnectPermission(JsonObject json, ResultCallback resultCallback) {
        LOGGER.info("[NCClientActions] onGetNCConnectPermission : ");

        if (json.get("data").getAsString().equals("success")) {
            handler.hasPermission = true;
            LOGGER.info("[NCClientActions] onGetNCConnectPermission : permission true");
        }
    }

    @Action
    public void receiveUpdate(JsonObject json, ResultCallback resultCallback) {
        LOGGER.info("[NCClientActions] receiveUpdate : ");
        List<OtherNCInfo> info = new ArrayList<OtherNCInfo>();

        if (json.has("empty")) {
            LOGGER.info("receive from NC ,do not have any cp.");
        } else {
            info = JsonUtil.fromJson(json.get("contracts").getAsString(), new TypeToken<List<OtherNCInfo>>() {
            }.getType());
        }

        LOGGER.info(JsonUtil.toJson(info));

        String id = json.get("id").getAsString();
        OtherNCProxy.instance.updateOtherNCInfo(id, info);

        String add = json.get("address").getAsString();
        LOGGER.info("[NCClientActinos] close connect " + add);
        OtherNCProxy.instance.connList.get(add).close();
    }

    public void sendMsg(String str) {
        try {
            handler.sendMsg(str);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
