package org.bdware.server.nodecenter;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.reflect.TypeToken;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufInputStream;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.codec.http.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.sc.db.KeyValueDBUtil;
import org.bdware.sc.util.ExceptionUtil;
import org.bdware.sc.util.JsonUtil;
import org.bdware.server.NodeCenterServer;
import org.bdware.server.action.Action;
import org.bdware.server.action.ActionExecutor;
import org.bdware.server.action.HttpResultCallback;
import org.bdware.server.http.HttpFileHandleAdapter;
import org.bdware.server.http.URIHandler;
import org.bdware.server.http.URIPath;
import org.bdware.server.permission.Role;
import org.zz.gmhelper.SM2Util;

import java.io.*;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import static io.netty.handler.codec.http.HttpResponseStatus.OK;

@Sharable
public class NCHttpHandler extends SimpleChannelInboundHandler<HttpObject> {
    private static final String PARAM_ACTION = "action";
    private static final String UNSUPPORTED_HTTP_METHOD = "{\"msg\":\"unsupported http method\"}";
    private static final String UNSUPPORTED_ACTION = "{\"msg\":\"unsupported action\"}";
    private static final Logger LOGGER = LogManager.getLogger(NCHttpHandler.class);
    private static final ActionExecutor<ResultCallback, JsonObject> actionExecutor =
            new ActionExecutor<ResultCallback, JsonObject>(
                    NodeCenterFrameHandler.executorService, new NodeCenterActions(null)) {

                @Override
                public boolean checkPermission(Action a, JsonObject arg, long per) {
                    boolean flag = a.httpAccess();
                    long val = a.userPermission(); // 使用者必须达到的permission
                    long permission = arg.get("permission").getAsLong();
                    LOGGER.debug("permission" + permission);
                    LOGGER.debug("htttttttttttttp--userpermission:" + val);
                    LOGGER.debug("htttttttttttttp--permission:" + per); // 现有的permission
                    boolean flag2 = false; // 启动http权限后应改为false
                    String status = "refuse";
                    String action = arg.get("action").getAsString();

                    if (val == 0) {
                        flag2 = true;
                    } else {
                        flag2 = (permission & val) == val;
                    }

                    if (flag && flag2) {
                        status = "accept";
                    }

                    String pubkey = "anonymity";
                    if (arg.has("pubKey")) {
                        pubkey = arg.get("pubKey").getAsString();
                    }
                    NodeCenterServer.nodeHttpLogDB.put(
                            action,
                            "{\"action\":\""
                                    + action
                                    + "\",\"pubKey\":\""
                                    + pubkey
                                    + "\",\"date\":"
                                    + System.currentTimeMillis()
                                    + "}");
                    LOGGER.debug("[NCHttpHandler] flag = " + flag + "  flag2 = " + flag2);
                    return flag && flag2;
                }
            };
    static TypeToken<Map<String, String>> token = new TypeToken<Map<String, String>>() {};
    private final HttpFileHandleAdapter fileAdapter;
    // public static ExecutorService executor = Executors.newFixedThreadPool(5);
    public AtomicInteger counter = new AtomicInteger(0);
    private URIHandler handler;

    public NCHttpHandler() {
        FileFilter fileFilter = pathname -> !pathname.getAbsolutePath().contains("..");
        fileAdapter =
                new HttpFileHandleAdapter(new File("./WebContent/").getAbsolutePath(), fileFilter) {
                    @URIPath("/")
                    public void loadFile(ChannelHandlerContext ctx, FullHttpRequest req)
                            throws Exception {
                        super.channelRead0(ctx, req);
                    }
                };
        handler = new URIHandler();
        handler.register(this);
        handler.register(fileAdapter);
        handler.register(FileActions.class);
        handler.printURIHandlers();
    }

    public static ActionExecutor<ResultCallback, JsonObject> getActionExecutor() {
        return actionExecutor;
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) {
        ctx.flush();
    }

    @URIPath({"/NodeCenter", "/SCIDE/SCExecutor", "/SCIDE/CMManager", "/SCIDE/SCManager"})
    public void parseGetAndHandle(ChannelHandlerContext ctx, FullHttpRequest req) {
        QueryStringDecoder decoderQuery = new QueryStringDecoder(req.uri());
        Map<String, List<String>> params = decoderQuery.parameters();
        JsonObject map = new JsonObject();
        for (String key : params.keySet()) {
            List<String> val = params.get(key);
            if (val != null) map.addProperty(key, val.get(0));
        }
        String uri = URLDecoder.decode(req.uri()).split("\\?")[1];
        int index = uri.lastIndexOf('&');
        String str = uri;
        if (index != -1) {
            str = uri.substring(0, index);
        }
        injectPermission(map, str);
        handleReq(map, ctx);
    }

    @URIPath(
            method = org.bdware.server.http.HttpMethod.POST,
            value = {"/NodeCenter", "/SCIDE/SCExecutor", "/SCIDE/CMManager", "/SCIDE/SCManager"})
    public void parsePostAndHandle(ChannelHandlerContext ctx, FullHttpRequest req)
            throws UnsupportedEncodingException {
        ByteBuf content = req.content();
        JsonObject map =
                JsonParser.parseReader(new InputStreamReader(new ByteBufInputStream(content)))
                        .getAsJsonObject();
        // FIXME 感觉不太对劲
        String uri = URLDecoder.decode(req.uri(), "utf-8").split("\\?")[1];
        int index = uri.lastIndexOf('&');
        String str = uri;
        if (index != -1) {
            str = uri.substring(0, index);
        }
        injectPermission(map, str);
        handleReq(map, ctx);
    }

    private JsonObject parseArgInQuery(FullHttpRequest request) {
        QueryStringDecoder decoderQuery = new QueryStringDecoder(request.uri());
        Map<String, List<String>> parameters = decoderQuery.parameters();
        JsonObject transformedParam = new JsonObject();
        for (String key : parameters.keySet()) {
            List<String> val = parameters.get(key);
            if (val != null) transformedParam.addProperty(key, val.get(0));
        }
        return transformedParam;
    }

    @URIPath(
            method = org.bdware.server.http.HttpMethod.GET,
            value = {"/doip"})
    public void handleDOIP(ChannelHandlerContext ctx, FullHttpRequest req) {
        ByteBuf content = req.content();
        JsonObject map = parseArgInQuery(req);
        // FIXME 感觉不太对劲
        String uri = URLDecoder.decode(req.uri()).split("\\?")[1];
        int index = uri.lastIndexOf('&');
        String str = uri;
        if (index != -1) {
            str = uri.substring(0, index);
        }
        injectPermission(map, str);

        DefaultFullHttpResponse response =
                new DefaultFullHttpResponse(
                        HttpVersion.HTTP_1_1,
                        OK,
                        Unpooled.wrappedBuffer(MetaIndexAction.search(map).getBytes()));
        ChannelFuture f = ctx.write(response);
        f.addListener(ChannelFutureListener.CLOSE);

        // TODO @fanbo Http查询接口，需支持各类查询，按owner/按关键词/....
    }

    @Override
    public void channelRead0(ChannelHandlerContext ctx, HttpObject msg) {
        //		logger.debug("[NCHttpHandler] TID:" + Thread.currentThread().getId() +
        // msg.toString());
        if (msg instanceof FullHttpRequest) {
            FullHttpRequest request = (FullHttpRequest) msg;
            handler.handle(ctx, request);
        }
    }

    public String getRole(String pubKey) {
        try {
            String ret =
                    KeyValueDBUtil.instance.getValue(
                            NCTables.ConfigDB.toString(), "__CenterManager__");
            if (ret != null && ret.length() > 0) {
                boolean isCenterManager = (ret.equals(pubKey)); // 表示此节点是否是平台管理员
                ret = KeyValueDBUtil.instance.getValue(NCTables.NodeUser.toString(), pubKey);
                String role = "";
                if (isCenterManager) {
                    role = "CenterManager,";
                }
                if (null != ret && !ret.isEmpty()) {
                    role += ret;
                }
                if (role.endsWith(",")) {
                    role = role.substring(0, role.length() - 1);
                } else if (role.isEmpty()) {
                    role = Role.Anonymous.name();
                }
                return role;
            }
            return Role.Anonymous.name();
        } catch (Exception e) {
            e.printStackTrace();
            return Role.Anonymous.name();
        }
    }

    private void injectPermission(JsonObject map, String str) {
        if (map.has("pubKey")) {
            String pubkey = map.get("pubKey").getAsString();
            boolean verify = SM2Util.plainStrVerify(pubkey, str, map.get("sign").getAsString());
            LOGGER.debug("[CMHttpHandler] HTTP POST 请求验签为 " + verify);

            if (verify) {
                // 查permission
                String ret = getRole(pubkey);
                long permission;
                if (ret != null && ret.length() > 0) {
                    permission = 0x86000d41L | Role.compoundValue(ret.split(","));
                } else {
                    permission = Role.compoundValue(ret.split(","));
                }
                map.addProperty("permission", permission + "");
                LOGGER.debug("[CMHttpHandler] http 请求查看用户权限为 : " + permission);
            } else {
                map.addProperty("permission", 0 + "");
            }
        } else {
            map.addProperty("permission", 0 + "");
        }
    }

    private void handleReq(final JsonObject map, final ChannelHandlerContext ctx) {
        try {
            map.addProperty("fromHttp", "true");
            String ret = null;
            String action = null;
            if (!map.has("action")) {
                ret = UNSUPPORTED_ACTION;
            } else {
                action = map.get("action").getAsString();
            }
            if (action != null) {
                HttpResultCallback cb;

                if (action.equals("takeScreenshot")) {
                    cb = new HttpResultCallback(ctx, map.get("callback").getAsString());
                    cb.setDecodeBase64();
                    cb.addHeader("Content-type", "image/png");

                } else if (action.equals("executeContract")) {
                    cb = new HttpResultCallback(ctx, map.get("callback").getAsString());
                    cb.addHeader("charset", "utf-8");
                    cb.addHeader("Content-type", "text/plain");

                } else {

                    if (map.has("callback"))
                        cb = new HttpResultCallback(ctx, map.get("callback").getAsString());
                    else cb = new HttpResultCallback(ctx, null);

                    cb.addHeader("charset", "utf-8");
                    cb.addHeader("Content-type", "application/json");
                }
                actionExecutor.handle(action, map, cb);
            } else {
                DefaultFullHttpResponse response =
                        new DefaultFullHttpResponse(
                                HttpVersion.HTTP_1_1, OK, Unpooled.wrappedBuffer(ret.getBytes()));
                ChannelFuture f = ctx.write(response);
                f.addListener(ChannelFutureListener.CLOSE);
            }
        } catch (IllegalArgumentException e) {
            DefaultFullHttpResponse response =
                    new DefaultFullHttpResponse(
                            HttpVersion.HTTP_1_1,
                            OK,
                            Unpooled.wrappedBuffer(e.getMessage().getBytes()));
            ChannelFuture f = ctx.write(response);
            f.addListener(ChannelFutureListener.CLOSE);

        } catch (Exception e) {
            Map<String, String> ret = new HashMap<>();
            ByteArrayOutputStream bo = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(bo));
            ret.put("msg", bo.toString());
            DefaultFullHttpResponse response =
                    new DefaultFullHttpResponse(
                            HttpVersion.HTTP_1_1,
                            OK,
                            Unpooled.wrappedBuffer(JsonUtil.toJson(ret).getBytes()));
            ChannelFuture f = ctx.write(response);
            f.addListener(ChannelFutureListener.CLOSE);
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        LOGGER.warn("catch exception in " + this + ": " + cause.getClass().getSimpleName());
        LOGGER.debug(ExceptionUtil.exceptionToString(cause));
        ctx.close();
    }
}
