package org.bdware.server.nodecenter;

import com.google.gson.JsonObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.sc.udp.UDPMessage;
import org.bdware.sc.udp.UDPNode;

import org.bdware.sc.util.JsonUtil;
import org.bdware.server.action.Action;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class NCUDPRunner extends Thread {
    private static final Logger LOGGER = LogManager.getLogger(NCUDPRunner.class);
    public static int mainPort;
    public ExecutorService executor = Executors.newFixedThreadPool(5);
    public int port;
    boolean ready = false;
    Map<Integer, org.bdware.sc.udp.UDPNode> id2IP;
    private DatagramSocket socket;

    public NCUDPRunner(Map<Integer, UDPNode> id2ip2) {
        // TODO Auto-generated constructor stub
        this.id2IP = id2ip2;
    }

    public void run() {
        //		TimerTask task = new TimerTask() {
        //
        //			@Override
        //			public void run(Timeout arg0) throws Exception {
        //				Set<Integer> nodes = new HashSet<Integer>();
        //				for (Integer i:id2IP.keySet()) {
        //					UDPNode node = id2IP.get(i);
        //					if (System.currentTimeMillis()-node.lastUpdatedTime<60*1000) {
        //						nodes.remove(i);
        //					}
        //				}
        //				for (Integer i:nodes)
        //					node.re
        //			}
        //		};
        id2IP = new HashMap<Integer, UDPNode>();
        int startPort = mainPort;
        while (true) {
            try {
                socket = new DatagramSocket(startPort);
                break;
            } catch (Exception e) {
                startPort++;
            }
        }
        this.port = startPort;
        ready = true;
        while (true) {
            try {
                final DatagramPacket request = new DatagramPacket(new byte[1024], 1024);
                socket.receive(request);
                final UDPMessage msg = UDPMessage.parse(request.getData(), request.getLength());
                executor.execute(() -> {
                    try {
                        onUDPMessage(msg, request);
                    } catch (Exception e) {
                        System.err.println("Excpetion in:" + port);
                        e.printStackTrace();
                    }
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void onUDPMessage(UDPMessage msg, DatagramPacket request) {
        switch (msg.type) {
            case shakeHand:
                LOGGER.debug(
                        "[NCUDPRunner] shakeHand:"
                                + request.getAddress().getHostAddress()
                                + ":"
                                + request.getPort());
                id2IP.put(msg.id, new UDPNode(request.getSocketAddress()));
            default:
        }
    }

    @Action
    public void listUDPNodes(JsonObject json, ResultCallback rc) {
        Map<String, String> ret = new HashMap<String, String>();
        ret.put("action", "onListUDPNodes");
        for (Integer i : id2IP.keySet()) {
            UDPNode node = id2IP.get(i);
            InetSocketAddress address = (InetSocketAddress) (node.addr);
            ret.put("id_" + i, address.getHostString() + ":" + address.getPort());
        }
        ret.put("udpPort", port + "");

        rc.onResult(JsonUtil.toJson(ret));
    }

    public String getUDPAddr(Integer udpid) {
        LOGGER.debug(udpid);
        UDPNode node = id2IP.get(udpid);
        if (node != null) {
            InetSocketAddress address = (InetSocketAddress) (node.addr);
            return address.getHostString() + ":" + address.getPort();
        }
        return null;
    }

    public void remove(String udpID) {
        id2IP.remove(Integer.valueOf(udpID));
    }
}
